@@ -1,4 +1,4 @@
-from bolinette.commands.commands import commands, command
 import bolinette.commands.run_server
 import bolinette.commands.init_db
 import bolinette.commands.init_docs
+import bolinette.commands.create_user
